@@ -25,6 +25,7 @@ This module provides the essential functionality. There are other packages in th
 Unofficial:
 
 * <http://hackage.haskell.org/package/microlens-aeson microlens-aeson> – a port of <http://hackage.haskell.org/package/lens-aeson lens-aeson>
+* <http://hackage.haskell.org/package/microlens-process microlens-process> - a port of <http://hackage.haskell.org/package/lens-process lens-process>
 
 -}
 module Lens.Micro
@@ -44,6 +45,8 @@ module Lens.Micro
   (?~),
   (<%~), (<<%~), (<<.~),
   mapped,
+  rewriteOf,
+  transformOf,
 
   -- * Getter: extracts a value from a structure
   -- $getters-note
@@ -112,7 +115,12 @@ import Data.Monoid
 import Data.Maybe
 import Data.Tuple
 import qualified Data.Foldable as F
+
+#if __GLASGOW_HASKELL__ >= 708
+import Data.Coerce
+#else
 import Unsafe.Coerce
+#endif
 
 #if MIN_VERSION_base(4,8,0)
 import Data.Function ((&))
@@ -304,57 +312,69 @@ over l f = runIdentity #. l (Identity #. f)
 {-# INLINE over #-}
 
 
--- | Increment the target(s) of a numerically valued 'Lens' or 'Traversal'.
---
--- >>> (a,b) & _1 +~ c
--- (a + c,b)
---
--- >>> (a,b) & both +~ c
--- (a + c,b + c)
---
--- >>> (1,2) & _2 +~ 1
--- (1,3)
---
--- >>> [(a,b),(c,d)] & traverse.both +~ e
--- [(a + e,b + e),(c + e,d + e)]
---
--- @
--- ('+~') :: 'Num' a => 'Lens'' s a      -> a -> s -> s
--- ('+~') :: 'Num' a => 'Traversal'' s a -> a -> s -> s
--- @
+{- |
+Increment the target(s) of a numerically valued 'Lens' or 'Traversal'.
+
+>>> (a,b) & _1 +~ c
+(a + c,b)
+
+>>> (a,b) & both +~ c
+(a + c,b + c)
+
+>>> (1,2) & _2 +~ 1
+(1,3)
+
+>>> [(a,b),(c,d)] & traverse.both +~ e
+[(a + e,b + e),(c + e,d + e)]
+
+@
+('+~') :: 'Num' a => 'Lens'' s a      -> a -> s -> s
+('+~') :: 'Num' a => 'Traversal'' s a -> a -> s -> s
+@
+
+@since 0.4.10
+-}
 (+~) :: Num a => ASetter s t a a -> a -> s -> t
 l +~ n = over l (+ n)
 {-# INLINE (+~) #-}
 
--- | Decrement the target(s) of a numerically valued 'Lens', or 'Traversal'.
---
--- >>> (a,b) & _1 -~ c
--- (a - c,b)
---
--- >>> (a,b) & both -~ c
--- (a - c,b - c)
---
--- >>> _1 -~ 2 $ (1,2)
--- (-1,2)
---
--- >>> mapped.mapped -~ 1 $ [[4,5],[6,7]]
--- [[3,4],[5,6]]
---
--- @
--- ('-~') :: 'Num' a => 'Lens'' s a      -> a -> s -> s
--- ('-~') :: 'Num' a => 'Traversal'' s a -> a -> s -> s
--- @
+infixr 4 +~
+
+{- |
+Decrement the target(s) of a numerically valued 'Lens', or 'Traversal'.
+
+>>> (a,b) & _1 -~ c
+(a - c,b)
+
+>>> (a,b) & both -~ c
+(a - c,b - c)
+
+>>> _1 -~ 2 $ (1,2)
+(-1,2)
+
+>>> mapped.mapped -~ 1 $ [[4,5],[6,7]]
+[[3,4],[5,6]]
+
+@
+('-~') :: 'Num' a => 'Lens'' s a      -> a -> s -> s
+('-~') :: 'Num' a => 'Traversal'' s a -> a -> s -> s
+@
+
+@since 0.4.10
+-}
 (-~) :: Num a => ASetter s t a a -> a -> s -> t
 l -~ n = over l (subtract n)
 {-# INLINE (-~) #-}
 
-
+infixr 4 -~
 
 {- |
 ('<>~') appends a value monoidally to the target.
 
 >>> ("hello", "goodbye") & both <>~ " world!"
 ("hello world!", "goodbye world!")
+
+@since 0.4.9
 -}
 (<>~) :: (Monoid a) => ASetter s t a a -> a -> s -> t
 (<>~) l a = over l (`mappend` a)
@@ -498,6 +518,30 @@ Simpler type signatures:
 
 infixr 4 <<.~
 
+{- |
+→ See <https://github.com/monadfix/microlens/pull/119#issuecomment-496004851 an example> on GitHub.
+
+Rewrite by applying a rule everywhere you can. Ensures that the rule cannot be applied anywhere in the result.
+
+Usually 'transformOf' is more appropriate, but 'rewriteOf' can give better compositionality. Given two single transformations @f@ and @g@, you can construct @\\a -> f a '<|>' g a@ which performs both rewrites until a fixed point.
+
+@since 0.4.11
+-}
+rewriteOf :: ASetter a b a b -> (b -> Maybe a) -> a -> b
+rewriteOf l f = go where
+  go = transformOf l (\x -> maybe x go (f x))
+{-# INLINE rewriteOf #-}
+
+{- |
+Transform every element by recursively applying a given 'ASetter' in a bottom-up manner.
+
+@since 0.4.11
+-}
+transformOf :: ASetter a b a b -> (b -> b) -> a -> b
+transformOf l f = go where
+  go = f . over l go
+{-# INLINE transformOf #-}
+
 -- Getting -----------------------------------------------------------------
 
 {- $getters-note
@@ -1015,8 +1059,13 @@ singular l afb s = case ins b of
   where
     Bazaar b = l sell s
     sell w = Bazaar ($ w)
+#if __GLASGOW_HASKELL__ >= 708
+    ins f = (coerce :: [Identity a] -> [a])
+              (getConst (f (\ra -> Const [Identity ra])))
+#else
     ins f = (unsafeCoerce :: [Identity a] -> [a])
               (getConst (f (\ra -> Const [Identity ra])))
+#endif
     unsafeOuts f = evalState (f (\_ -> state (unconsWithDefault fakeVal)))
       where fakeVal = error "unsafeOuts: not enough elements were supplied"
     unconsWithDefault d []     = (d,[])
@@ -1413,8 +1462,10 @@ instance (Monad m) => Monad (StateT s m) where
         ~(a, s') <- runStateT m s
         runStateT (k a) s'
     {-# INLINE (>>=) #-}
+#if !MIN_VERSION_base(4,13,0)
     fail str = StateT $ \ _ -> fail str
     {-# INLINE fail #-}
+#endif
 
 #if MIN_VERSION_base(4,9,0)
 instance (Fail.MonadFail m) => Fail.MonadFail (StateT s m) where
