@@ -10,8 +10,9 @@
 {-# LANGUAGE FunctionalDependencies #-}
 {-# LANGUAGE ConstraintKinds #-}
 {-# LANGUAGE ImplicitParams #-}
-{-# LANGUAGE Unsafe #-}
 
+-- Note: this module is marked 'Unsafe' because it exports (#.), which is essentially 'coerce', and Data.Coerce is marked 'Unsafe' in base. As per <https://github.com/ekmett/lens/issues/661>, this is an issue for 'lens' as well but they have opted for 'Trustworthy' instead.
+{-# LANGUAGE Unsafe #-}
 
 {- |
 Module      :  Lens.Micro.Internal
@@ -156,6 +157,10 @@ noEffect = phantom (pure ())
 -- Data.Profunctor.Unsafe
 ------------------------------------------------------------------------------
 
+-- Note: 'lens' defines a type-restricted version of (#.) to work around a
+-- bug, but our version is restricted enough that we don't need it. See
+-- <https://github.com/ekmett/lens/commit/cde2fc39c0dba413d1a6f814b47bd14431a5e339>
+
 #if __GLASGOW_HASKELL__ >= 708
 ( #. ) :: Coercible c b => (b -> c) -> (a -> b) -> (a -> c)
 ( #. ) _ = coerce (\x -> x :: b) :: forall a b. Coercible b a => a -> b
@@ -198,6 +203,7 @@ You can use 'each' with these things:
 'each' :: 'Traversal' [a] [b] a b
 
 'each' :: 'Traversal' ('Maybe' a) ('Maybe' b) a b
+'each' :: 'Traversal' ('Either' a a) ('Either' b b) a b  -- since 0.4.11
 
 'each' :: 'Traversal' (a,a) (b,b) a b
 'each' :: 'Traversal' (a,a,a) (b,b,b) a b
@@ -239,12 +245,22 @@ instance Each (Maybe a) (Maybe b) a b where
   each = traverse
   {-# INLINE each #-}
 
+{- |
+@since 0.4.11
+-}
+instance (a~a', b~b') => Each (Either a a') (Either b b') a b where
+  each f (Left a)   = Left <$> f a
+  each f (Right a ) = Right <$> f a
+  {-# INLINE each #-}
+
 #if __GLASGOW_HASKELL__ >= 800
 instance Each (NonEmpty a) (NonEmpty b) a b where
   each = traversed
   {-# INLINE each #-}
 #endif
 
+-- NOTE: when adding new instances of 'Each', update the docs for 'each'.
+
 type family Index (s :: *) :: *
 
 type family IxValue (m :: *) :: *
